@@ -30,6 +30,7 @@
 
 #include "file-data.h"
 #include "file-utils.h"
+#include "glib-utils.h"
 #include "fr-command.h"
 #include "fr-command-rar.h"
 
@@ -171,7 +172,7 @@ add_password_arg (FRCommand	*comm,
 	if (password != NULL && password[0] != '\0') {
 		char *arg, *e_password;
 		
-		e_password = escape_str (password, "\"");
+		e_password = escape_str (password, "\"*?[]'`()$!;");
 		g_assert (e_password != NULL);  /* since password is non-NULL */
 		
 		arg = g_strconcat ("-p\"", e_password, "\"", NULL);
@@ -363,9 +364,12 @@ static void
 fr_command_rar_handle_error (FRCommand   *comm, 
 			     FRProcError *error)
 {
-	if ((error->type == FR_PROC_ERROR_GENERIC) 
-	    && (error->status <= 1))
-		error->type = FR_PROC_ERROR_NONE;
+	if (error->type == FR_PROC_ERROR_GENERIC) {
+		if (error->status <= 1)
+			error->type = FR_PROC_ERROR_NONE;
+		else if (error->status == 3)
+			error->type = FR_PROC_ERROR_ASK_PASSWORD;
+	}
 }
 
 
@@ -392,6 +396,8 @@ fr_command_rar_class_init (FRCommandRarClass *class)
 static void 
 fr_command_rar_init (FRCommand *comm)
 {
+	comm->file_type = FR_FILE_TYPE_RAR;
+
 	comm->propAddCanUpdate             = TRUE; 
 	comm->propAddCanReplace            = TRUE; 
 	comm->propExtractCanAvoidOverwrite = TRUE;
