@@ -31,6 +31,7 @@
 
 #include "file-data.h"
 #include "file-utils.h"
+#include "glib-utils.h"
 #include "fr-command.h"
 #include "fr-command-rpm.h"
 
@@ -182,6 +183,8 @@ fr_command_rpm_class_init (FRCommandRpmClass *class)
 static void 
 fr_command_rpm_init (FRCommand *comm)
 {
+	comm->file_type = FR_FILE_TYPE_RPM;
+
 	comm->propCanModify                = FALSE;
 	comm->propAddCanUpdate             = FALSE;
 	comm->propAddCanReplace            = FALSE;
