@@ -21,12 +21,18 @@
  */
 
 #include <config.h>
-#include <gnome.h>
+#include <string.h>
+
+#include <libgnome/gnome-config.h>
+#include <libgnomeui/gnome-client.h>
+#include <libgnomeui/gnome-app.h>
+#include <libgnomeui/gnome-ui-init.h>
 #include <libgnomeui/gnome-window-icon.h>
 #include <libgnomevfs/gnome-vfs-init.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <glade/glade.h>
 #include "file-utils.h"
+#include "glib-utils.h"
 #include "fr-process.h"
 #include "fr-stock.h"
 #include "gconf-utils.h"
@@ -43,18 +49,87 @@ static void     init_session        (char **argv);
 static gboolean session_is_restored (void);
 static gboolean load_session        (void);
 
-
 GList        *window_list = NULL;
 GList        *viewer_list = NULL;
 GList        *command_list = NULL;
 gint          force_directory_creation;
+GHashTable   *programs_cache = NULL;
 
 static gchar *add_to = NULL;
 static gint   add;
 static gchar *extract_to = NULL;
 static gint   extract;
+static gint   extract_here;
 static gchar *default_url = NULL;
 
+FRFileTypeDescription file_type_desc[] = {
+	{ FR_FILE_TYPE_AR,           ".ar",      "application/x-ar", N_("Ar (.ar)") },
+	{ FR_FILE_TYPE_DEB,          ".deb",     "application/x-deb", NULL },
+	{ FR_FILE_TYPE_ARJ,          ".arj",     "application/x-arj", N_("Arj (.arj)") },
+	{ FR_FILE_TYPE_BZIP,         ".bz",      "application/x-bzip", NULL },
+	{ FR_FILE_TYPE_BZIP2,        ".bz2",     "application/x-bzip", NULL },
+	{ FR_FILE_TYPE_COMPRESS,     ".Z",       "application/x-compress", NULL },
+	{ FR_FILE_TYPE_ISO,          ".iso",     "application/x-cd-image", NULL },
+	{ FR_FILE_TYPE_EAR,          ".ear",     "application/x-ear", N_("Ear (.ear)") },
+	{ FR_FILE_TYPE_GZIP,         ".gz",      "application/x-gzip", NULL},
+	{ FR_FILE_TYPE_JAR,          ".jar",     "application/x-jar", N_("Jar (.jar)")},
+	{ FR_FILE_TYPE_LHA,          ".lzh",     "application/x-lha", N_("Lha (.lzh)") },
+	{ FR_FILE_TYPE_LZOP,         ".lzo",     "application/x-lzop", NULL },
+	{ FR_FILE_TYPE_RAR,          ".rar",     "application/x-rar", N_("Rar (.rar)") },
+	{ FR_FILE_TYPE_RPM,          ".rpm",     "application/x-rpm", NULL },
+	{ FR_FILE_TYPE_TAR,          ".tar",     "application/x-tar", N_("Tar uncompressed (.tar)") },
+	{ FR_FILE_TYPE_TAR_BZ,       ".tar.bz",  "application/x-bzip-compressed-tar", N_("Tar compressed with bzip (.tar.bz)") },
+	{ FR_FILE_TYPE_TAR_BZ2,      ".tar.bz2", "application/x-bzip-compressed-tar", N_("Tar compressed with bzip2 (.tar.bz2)") },
+	{ FR_FILE_TYPE_TAR_GZ,       ".tar.gz",  "application/x-compressed-tar", N_("Tar compressed with gzip (.tar.gz)") },
+	{ FR_FILE_TYPE_TAR_LZOP,     ".tar.lzo", "application/x-lzop-compressed-tar", N_("Tar compressed with lzop (.tar.lzo)") },
+	{ FR_FILE_TYPE_TAR_COMPRESS, ".tar.Z",   "application/x-compressed-tar", N_("Tar compressed with compress (.tar.Z)") },
+	{ FR_FILE_TYPE_STUFFIT,      ".sit",     "application/x-stuffit", NULL },
+	{ FR_FILE_TYPE_WAR,          ".war",     "application/zip", N_("War (.war)") },
+	{ FR_FILE_TYPE_ZIP,          ".zip",     "application/zip", N_("Zip (.zip)") },
+	{ FR_FILE_TYPE_ZOO,          ".zoo",     "application/x-zoo", N_("Zoo (.zoo)") },
+	{ FR_FILE_TYPE_7ZIP,         ".7z",      "application/x-7z-compressed", N_("7-Zip (.7z)") }
+};
+
+
+FRFileType single_file_save_type[32];
+FRFileType save_type[32];
+FRFileType open_type[32];
+
+
+FRCommandDescription command_desc[] = {
+	{ "tar",        TRUE,  TRUE,  TRUE,  FR_FILE_TYPE_TAR },
+	{ "zip",        TRUE,  TRUE,  TRUE,  FR_FILE_TYPE_ZIP },
+	{ "unzip",      TRUE,  FALSE, TRUE,  FR_FILE_TYPE_ZIP },
+	{ "rar",        TRUE,  TRUE,  TRUE,  FR_FILE_TYPE_RAR },
+	{ "gzip",       TRUE,  TRUE,  FALSE, FR_FILE_TYPE_GZIP },
+	{ "bzip2",      TRUE,  TRUE,  FALSE, FR_FILE_TYPE_BZIP2 },
+	{ "ar",         TRUE,  TRUE,  TRUE,  FR_FILE_TYPE_AR },
+	{ "ar",         TRUE,  FALSE, TRUE,  FR_FILE_TYPE_DEB },
+	{ "arj",        TRUE,  TRUE,  TRUE,  FR_FILE_TYPE_ARJ },
+	{ "bzip2",      TRUE,  FALSE, FALSE, FR_FILE_TYPE_BZIP },
+	{ "compress",   TRUE,  TRUE,  FALSE, FR_FILE_TYPE_COMPRESS },
+	{ "isoinfo",    TRUE,  FALSE, TRUE,  FR_FILE_TYPE_ISO },
+	{ "zip",        TRUE,  TRUE,  TRUE,  FR_FILE_TYPE_EAR },
+	{ "zip",        TRUE,  TRUE,  TRUE,  FR_FILE_TYPE_JAR },
+	{ "lha",        TRUE,  TRUE,  TRUE,  FR_FILE_TYPE_LHA },
+	{ "lzop",       TRUE,  TRUE,  FALSE, FR_FILE_TYPE_LZOP },
+	{ "rpm2cpio",   TRUE,  FALSE, TRUE,  FR_FILE_TYPE_RPM },
+	{ "uncompress", TRUE,  FALSE, FALSE, FR_FILE_TYPE_COMPRESS },
+	{ "unstuff",    TRUE,  FALSE, FALSE, FR_FILE_TYPE_STUFFIT },
+	{ "zip",        TRUE,  TRUE,  TRUE,  FR_FILE_TYPE_WAR },
+	{ "zoo",        TRUE,  TRUE,  TRUE,  FR_FILE_TYPE_ZOO },
+	{ "7za",        TRUE,  TRUE,  TRUE,  FR_FILE_TYPE_7ZIP }
+};
+
+FRCommandDescription tar_command_desc[] = {
+	{ "gzip",      TRUE,  TRUE,  TRUE,  FR_FILE_TYPE_TAR_GZ },
+	{ "bzip2",     TRUE,  TRUE,  TRUE,  FR_FILE_TYPE_TAR_BZ2 },
+	{ "bzip",      FALSE, TRUE,  TRUE,  FR_FILE_TYPE_TAR_BZ },
+	{ "lzop",      TRUE,  TRUE,  TRUE,  FR_FILE_TYPE_TAR_LZOP },
+	{ "compress",  TRUE,  TRUE,  TRUE,  FR_FILE_TYPE_TAR_COMPRESS }
+};
+
+
 struct poptOption options[] = {
 	{ "add-to", 'a', POPT_ARG_STRING, &add_to, 0,
 	  N_("Add files to the specified archive and quit the program"),
@@ -72,6 +147,10 @@ struct poptOption options[] = {
           N_("Extract archives asking the destination folder and quit the program"),
           0 },
 
+	{ "extract-here", 'h', POPT_ARG_NONE, &extract_here, 0,
+          N_("Extract archives using the archive name as destination folder and quit the program"),
+          0 },
+
 	{ "default-dir", 0, POPT_ARG_STRING, &default_url, 0,
 	  N_("Default folder to use for the '--add' and '--extract' commands"),
 	  N_("FOLDER") },
@@ -86,11 +165,31 @@ struct poptOption options[] = {
 
 /* -- Main -- */
 
-int main (int argc, char **argv)
+
+static guint startup_id = 0;
+static poptContext pctx = NULL; 
+
+
+static gboolean
+startup_cb (gpointer data)
+{
+	g_source_remove (startup_id);
+	startup_id = 0;
+
+	initialize_data ();
+	prepare_app (pctx);
+	poptFreeContext (pctx);
+	pctx = NULL;
+
+	return FALSE;
+}
+
+
+int 
+main (int argc, char **argv)
 {
 	GnomeProgram *program;
 	GValue value = { 0 };
-	poptContext pctx;
 
 	bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
@@ -99,13 +198,16 @@ int main (int argc, char **argv)
 	program = gnome_program_init ("file-roller", VERSION,
 				      LIBGNOMEUI_MODULE, argc, argv,
 				      GNOME_PARAM_POPT_TABLE, options,
-				      GNOME_PARAM_HUMAN_READABLE_NAME, _("File Roller"),
+				      GNOME_PARAM_HUMAN_READABLE_NAME, _("Archive Manager"),
 				      GNOME_PARAM_APP_PREFIX, FR_PREFIX,
                                       GNOME_PARAM_APP_SYSCONFDIR, FR_SYSCONFDIR,
                                       GNOME_PARAM_APP_DATADIR, FR_DATADIR,
                                       GNOME_PARAM_APP_LIBDIR, FR_LIBDIR,
 				      NULL);
 
+	g_set_application_name (_("File Roller"));
+	gtk_window_set_default_icon_name ("file-roller");
+
 	g_object_get_property (G_OBJECT (program),
 			       GNOME_PARAM_POPT_CONTEXT,
 			       g_value_init (&value, G_TYPE_POINTER));
@@ -114,15 +216,11 @@ int main (int argc, char **argv)
 	if (! gnome_vfs_init ()) 
                 g_error ("Cannot initialize the Virtual File System.");
 
-	glade_gnome_init ();
+	glade_init ();
 	fr_stock_init ();
 	init_session (argv);
-	initialize_data ();
-	prepare_app (pctx);
-	poptFreeContext (pctx);
-
+	startup_id = g_idle_add (startup_cb, NULL);
 	gtk_main ();
-
 	release_data ();
 
 	return 0;
@@ -134,10 +232,13 @@ int main (int argc, char **argv)
 static void 
 initialize_data ()
 {
-	gtk_window_set_default_icon_name ("file-roller");
-
 	eel_gconf_monitor_add ("/apps/file-roller");
 	eel_gconf_monitor_add (PREF_NAUTILUS_CLICK_POLICY);
+
+	programs_cache = g_hash_table_new_full (g_str_hash,
+						g_str_equal,
+						g_free,
+						NULL);
 }
 
 /* Free application data. */
@@ -204,6 +305,8 @@ command_done (CommandData *cdata)
 static void 
 release_data ()
 {
+	g_hash_table_destroy (programs_cache);
+
 	eel_global_client_free ();
 
 	while (viewer_list != NULL) {
@@ -310,6 +413,43 @@ migrate_to_new_directories (void)
 }
 
 
+static void 
+compute_supported_archive_types (void)
+{
+	int i, j;
+	int sf_i = 0, s_i = 0, o_i = 0;
+
+	for (i = 0; i < G_N_ELEMENTS (command_desc); i++) {
+		FRCommandDescription com = command_desc[i];
+
+		if (!is_program_in_path (com.command)) 
+			continue;
+
+		if (strcmp (com.command, "tar") == 0) 
+			for (j = 0; j < G_N_ELEMENTS (tar_command_desc); j++) {
+				FRCommandDescription com2 = tar_command_desc[j];
+				
+				if (!is_program_in_path (com2.command)) 
+					continue;
+				open_type[o_i++] = com2.file_type;
+				save_type[s_i++] = com2.file_type;
+				single_file_save_type[sf_i++] = com2.file_type;
+			}
+
+		if (com.can_open)
+			open_type[o_i++] = com.file_type;
+		if (com.can_save && com.support_many_files)
+			save_type[s_i++] = com.file_type;
+		if (com.can_save)
+			single_file_save_type[sf_i++] = com.file_type;
+	}
+
+	open_type[o_i++] = FR_FILE_TYPE_NULL;
+	save_type[s_i++] = FR_FILE_TYPE_NULL;
+	single_file_save_type[sf_i++] = FR_FILE_TYPE_NULL;
+}
+
+
 static void 
 prepare_app (poptContext pctx)
 {
@@ -332,6 +472,8 @@ prepare_app (poptContext pctx)
 	if (eel_gconf_get_boolean (PREF_MIGRATE_DIRECTORIES, TRUE))
                 migrate_to_new_directories ();
 
+	compute_supported_archive_types();
+
 	if (session_is_restored ()) {
 		load_session ();
 		return;
@@ -348,7 +490,7 @@ prepare_app (poptContext pctx)
 
 	default_dir = get_path_from_url (default_url);
 
-	if (extract_to != NULL) {
+	if ((extract_here == 0) && (extract_to != NULL)) {
 		if (g_path_is_absolute (extract_to))
 			extract_to_path = get_path_from_url (extract_to);
 		else {
@@ -397,7 +539,9 @@ prepare_app (poptContext pctx)
 		window_archive__quit (window);
 		window_batch_mode_start (window);
 
-	} else if ((extract_to != NULL) || (extract == 1)) { /* Extract all archives. */
+	} else if ((extract_to != NULL) 
+		   || (extract == 1) 
+		   || (extract_here == 1)) { /* Extract all archives. */
 		FRWindow   *window;
 		const char *archive;
 		
@@ -405,10 +549,21 @@ prepare_app (poptContext pctx)
 		if (default_dir != NULL)
 			window_set_default_dir (window, default_dir, TRUE);
 
-		while ((archive = poptGetArg (pctx)) != NULL) 
+		while ((archive = poptGetArg (pctx)) != NULL) {
+			if (extract_here == 1) {
+				g_free (extract_to_path);
+				extract_to_path = g_strconcat (archive, "_FILES", NULL);
+			}
+				
 			window_archive__open_extract (window, 
 						      archive, 
 						      extract_to_path);
+
+			if (extract_here == 1) {
+				g_free (extract_to_path);
+				extract_to_path = NULL;
+			}
+		}
 		window_archive__quit (window);
 		window_batch_mode_start (window);
 
